@@ -2,7 +2,7 @@
 This file will contain general functions to help complete processes.
 """
 # Initial Config
-from datetime import date, datetime
+from datetime import date, datetime, timedelta
 from io import StringIO
 import google.auth
 from google.cloud import storage
@@ -471,6 +471,72 @@ class GCP:
             list(sub_folders_times_dict.values()).index(latest)
         ]
 
+    @staticmethod
+    def generate_download_signed_url_v4(bucket_name: str,
+                                        blob_name: str,
+                                        json_path: str) -> str:
+        """
+        Generates a temporary URL to a GCP object specified from the bucket
+        and blob name as well as the assigning the relevant credentials from
+        the json_path param.
+
+        Parameters
+        ----------
+        bucket_name: str, the name of the bucket.
+        blob_name: str, the name of the file to be retrieved.
+        json_path: str, the path of the credentials.
+
+        Returns
+        -------
+        A string with the latest sub folder within a bucket and prefix.
+
+        """
+        # Initialise the client
+        storage_client = storage.Client.from_service_account_json(json_path)
+
+        # Initialise the bucket and the blob
+        bucket = storage_client.bucket(bucket_name)
+        blob = bucket.blob(blob_name)
+
+        # Generate the signed URL, only valid for a certain length of time
+        url = blob.generate_signed_url(
+            version="v4",
+            expiration=timedelta(minutes=15),
+            method="GET"
+
+        )
+
+        return url
+
+    @staticmethod
+    def upload_dataframe_to_gcp_bucket(df: pd.DataFrame,
+                                       bucket_name: str,
+                                       blob_name: str,
+                                       json_path: str
+                                       ):
+        """
+
+        Parameters
+        ----------
+        df: pd.DataFrame, the dataframe to be uploaded.
+        bucket_name: str, the name of the bucket.
+        blob_name: str, the name of the file.
+        json_path: str, the path of the credentials.
+
+        Returns
+        -------
+        An uploaded dataframe to a GCP bucket.
+
+        """
+        # Initialise the client
+        storage_client = storage.Client.from_service_account_json(json_path)
+
+        # Initialise the bucket and the blob
+        bucket = storage_client.bucket(bucket_name)
+        blob = bucket.blob(blob_name)
+
+        blob.upload_from_string(df.to_parquet(), 'text/parquet')
+
 
 class CloudSQL:
     """
